package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.ConferenciaPendencia;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.PantallaUtileria;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderConfPendenciaComissao;

@Controller("busquedaTipoConfCondComissaoController")
@Scope("prototype")
public class BusquedaTipoConfPendComissaoController extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<ConferenciaPendencia> pageListWrapperConferenciaPendencia;

	private MyTextbox txtDescricao;
	private Paging pagingTipoConfPendComissao;

	private MyListbox confPendComissaoList;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		confPendComissaoList.setItemRenderer(new RenderConfPendenciaComissao());
		confPendComissaoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event arg0) throws Exception {
				ConferenciaPendencia conferenciaPendencia = (ConferenciaPendencia) confPendComissaoList.getSelected();
				verTipoConfPendencia(conferenciaPendencia);
			}
		});
		atualizar();
	}

	public void onClick$btnNovo() {
		verTipoConfPendencia(new ConferenciaPendencia());
	}

	public void onClick$btnPesquisa() {
		atualizar();
	}

	public void onClick$btnRefresh() {
		atualizar();
	}

	private void atualizar() {
		HibernateSearchObject<ConferenciaPendencia> conferenciaPendenciaBusqueda = new HibernateSearchObject<ConferenciaPendencia>(ConferenciaPendencia.class, pagingTipoConfPendComissao.getPageSize());

		String descricao = txtDescricao.getValue();
		if (!descricao.isEmpty()) {
			conferenciaPendenciaBusqueda.addFilterILike("descpendencia", descricao.trim().concat("%"));
		}

		conferenciaPendenciaBusqueda.addSortAsc("descpendencia");
		conferenciaPendenciaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		pageListWrapperConferenciaPendencia.init(conferenciaPendenciaBusqueda, confPendComissaoList, pagingTipoConfPendComissao);
		if (confPendComissaoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaCoeficienteTarifaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	private void verTipoConfPendencia(ConferenciaPendencia conferenciaPendencia) {
		Map<String, Object> args = new HashMap<String, Object>();
		args.put("listBoxConfPendComissao", confPendComissaoList);
		args.put("confPendComissao", conferenciaPendencia);

		PantallaUtileria.openWindow("/gui/catalogos/editarTipoConfPendComissao.zul",
				Labels.getLabel("editarTipoConfPendComissao.window.title"),
				args, desktop);

	}

}
